# tifffile/__init__.py

from .tifffile import *
from .tifffile import __all__, __doc__, __version__, main

# constants are repeated for documentation

__version__ = __version__
"""Tifffile version string."""
